@@ -8,3 +8,5 @@ from descope.common import (
 )
 from descope.descope_client import DescopeClient
 from descope.exceptions import AuthException
+from descope.management.sso_settings import RoleMapping
+from descope.management.user import UserTenants
