@@ -1,10 +1,12 @@
 import json
 import os
+import platform
 import re
 from threading import Lock
 from typing import Tuple
 
 import jwt
+import pkg_resources
 import requests
 from email_validator import EmailNotValidError, validate_email
 from jwt.exceptions import ExpiredSignatureError
@@ -36,6 +38,7 @@ class Auth:
         project_id: str = None,
         public_key: str = None,
         skip_verify: bool = False,
+        management_key: str = None,
     ):
         self.lock_public_keys = Lock()
         # validate project id
@@ -56,6 +59,10 @@ class Auth:
 
         self.base_url = os.getenv("DESCOPE_BASE_URI", None) or DEFAULT_BASE_URL
 
+        if not management_key:
+            management_key = os.getenv("DESCOPE_MANAGEMENT_KEY", None)
+        self.management_key = management_key
+
         if not public_key:
             public_key = os.getenv("DESCOPE_PUBLIC_KEY", None)
 
@@ -86,12 +93,16 @@ class Auth:
             )
         return response
 
-    def do_post(self, uri: str, body: dict, pswd: str = None) -> requests.Response:
+    def do_post(
+        self, uri: str, body: dict, params=None, pswd: str = None
+    ) -> requests.Response:
         response = requests.post(
             f"{self.base_url}{uri}",
             headers=self._get_default_headers(pswd),
             data=json.dumps(body),
+            allow_redirects=False,
             verify=self.secure,
+            params=params,
         )
         if not response.ok:
             raise AuthException(
@@ -107,8 +118,8 @@ class Auth:
                 "exchange code is empty",
             )
 
-        params = Auth._compose_exchange_params(code)
-        response = self.do_get(uri, params, False)
+        body = Auth._compose_exchange_body(code)
+        response = self.do_post(uri, body, None)
         resp = response.json()
         jwt_response = self.generate_jwt_response(
             resp, response.cookies.get(REFRESH_SESSION_COOKIE_NAME, None)
@@ -129,7 +140,7 @@ class Auth:
             if not user.get("email", None):
                 user["email"] = identifier
             try:
-                validate_email(user["email"])
+                validate_email(email=user["email"], check_deliverability=False)
                 return True
             except EmailNotValidError:
                 return False
@@ -190,7 +201,7 @@ class Auth:
             )
 
         try:
-            validate_email(email)
+            validate_email(email=email, check_deliverability=False)
         except EmailNotValidError as ex:
             raise AuthException(
                 400, ERROR_TYPE_INVALID_ARGUMENT, f"Invalid email address: {ex}"
@@ -217,24 +228,19 @@ class Auth:
 
     def refresh_token(self, refresh_token: str) -> dict:
         uri = Auth._compose_refresh_token_url()
-        response = self.do_get(uri, None, None, refresh_token)
+        response = self.do_post(uri, {}, None, refresh_token)
 
         resp = response.json()
         return self.generate_jwt_response(resp, refresh_token)
 
     def exchange_access_key(self, access_key: str) -> dict:
         uri = Auth._compose_exchange_access_key_url()
-        server_response = self.do_get(uri, None, None, access_key)
+        server_response = self.do_post(uri, {}, None, access_key)
         json = server_response.json()
-        result = {
-            SESSION_TOKEN_NAME: self._validate_token(json.get("sessionJwt", "")),
-            "keyId": json.get("keyId", ""),
-            "exp": json.get("expiration", 0),
-        }
-        return result
+        return self._generate_auth_info(json, None, False)
 
     @staticmethod
-    def _compose_exchange_params(code: str) -> dict:
+    def _compose_exchange_body(code: str) -> dict:
         return {"code": code}
 
     @staticmethod
@@ -321,7 +327,7 @@ class Auth:
                 # just continue to the next key
                 pass
 
-    def adjust_properties(self, jwt_response: dict):
+    def adjust_properties(self, jwt_response: dict, user_jwt: bool):
         # Save permissions, roles and tenants info from Session token or from refresh token on the json top level
         if jwt_response.get(SESSION_TOKEN_NAME, None):
             jwt_response["permissions"] = jwt_response.get(SESSION_TOKEN_NAME).get(
@@ -345,18 +351,29 @@ class Auth:
             )
 
         # Save the projectID also in the dict top level
-        jwt_response["projectId"] = jwt_response.get(SESSION_TOKEN_NAME, {}).get(
+        issuer = jwt_response.get(SESSION_TOKEN_NAME, {}).get(
             "iss", None
-        ) or jwt_response.get(REFRESH_SESSION_TOKEN_NAME, {}).get("iss", None)
+        ) or jwt_response.get(REFRESH_SESSION_TOKEN_NAME, {}).get("iss", "")
+        jwt_response["projectId"] = issuer.rsplit("/")[
+            -1
+        ]  # support both url issuer and project ID issuer
 
-        # Save the userID also in the dict top level
-        jwt_response["userId"] = jwt_response.get(SESSION_TOKEN_NAME, {}).get(
-            "sub", None
-        ) or jwt_response.get(REFRESH_SESSION_TOKEN_NAME, {}).get("sub", None)
+        if user_jwt:
+            # Save the userID also in the dict top level
+            jwt_response["userId"] = jwt_response.get(SESSION_TOKEN_NAME, {}).get(
+                "sub", None
+            ) or jwt_response.get(REFRESH_SESSION_TOKEN_NAME, {}).get("sub", None)
+        else:
+            # Save the AccessKeyID also in the dict top level
+            jwt_response["keyId"] = jwt_response.get(SESSION_TOKEN_NAME, {}).get(
+                "sub", None
+            )
 
         return jwt_response
 
-    def _generate_auth_info(self, response_body: dict, refresh_token: str) -> dict:
+    def _generate_auth_info(
+        self, response_body: dict, refresh_token: str, user_jwt: bool
+    ) -> dict:
         jwt_response = {}
         st_jwt = response_body.get("sessionJwt", "")
         if st_jwt:
@@ -369,19 +386,20 @@ class Auth:
         elif rt_jwt:
             jwt_response[REFRESH_SESSION_TOKEN_NAME] = self._validate_token(rt_jwt)
 
-        jwt_response = self.adjust_properties(jwt_response)
+        jwt_response = self.adjust_properties(jwt_response, user_jwt)
 
-        jwt_response[COOKIE_DATA_NAME] = {
-            "exp": response_body.get("cookieExpiration", 0),
-            "maxAge": response_body.get("cookieMaxAge", 0),
-            "domain": response_body.get("cookieDomain", ""),
-            "path": response_body.get("cookiePath", "/"),
-        }
+        if user_jwt:
+            jwt_response[COOKIE_DATA_NAME] = {
+                "exp": response_body.get("cookieExpiration", 0),
+                "maxAge": response_body.get("cookieMaxAge", 0),
+                "domain": response_body.get("cookieDomain", ""),
+                "path": response_body.get("cookiePath", "/"),
+            }
 
         return jwt_response
 
     def generate_jwt_response(self, response_body: dict, refresh_cookie: str) -> dict:
-        jwt_response = self._generate_auth_info(response_body, refresh_cookie)
+        jwt_response = self._generate_auth_info(response_body, refresh_cookie, True)
 
         jwt_response["user"] = response_body.get("user", {})
         jwt_response["firstSeen"] = response_body.get("firstSeen", True)
@@ -391,6 +409,16 @@ class Auth:
         headers = {}
         headers["Content-Type"] = "application/json"
 
+        headers["x-descope-sdk-name"] = "python"
+
+        try:
+            headers["x-descope-sdk-python-version"] = platform.python_version()
+            headers["x-descope-sdk-version"] = pkg_resources.get_distribution(
+                "descope"
+            ).version
+        except Exception:
+            pass
+
         bearer = self.project_id
         if pswd:
             bearer = f"{self.project_id}:{pswd}"
