@@ -1,5 +1,5 @@
 from descope.auth import Auth
-from descope.common import EndpointsV1, OAuthProviders
+from descope.common import EndpointsV1, LoginOptions, validateRefreshTokenProvided
 from descope.exceptions import ERROR_TYPE_INVALID_ARGUMENT, AuthException
 
 
@@ -9,7 +9,13 @@ class OAuth:
     def __init__(self, auth: Auth):
         self._auth = auth
 
-    def start(self, provider: str, return_url: str = "") -> dict:
+    def start(
+        self,
+        provider: str,
+        return_url: str = "",
+        loginOptions: LoginOptions = None,
+        refreshToken: str = None,
+    ) -> dict:
         """ """
         if not self._verify_provider(provider):
             raise AuthException(
@@ -18,9 +24,13 @@ class OAuth:
                 f"Unknown OAuth provider: {provider}",
             )
 
+        validateRefreshTokenProvided(loginOptions, refreshToken)
+
         uri = EndpointsV1.oauthStart
         params = OAuth._compose_start_params(provider, return_url)
-        response = self._auth.do_get(uri, params, False)
+        response = self._auth.do_post(
+            uri, loginOptions.__dict__ if loginOptions else {}, params, refreshToken
+        )
 
         return response.json()
 
@@ -32,11 +42,7 @@ class OAuth:
     def _verify_provider(oauth_provider: str) -> str:
         if oauth_provider == "" or oauth_provider is None:
             return False
-
-        if oauth_provider in OAuthProviders:
-            return True
-        else:
-            return False
+        return True
 
     @staticmethod
     def _compose_start_params(provider: str, returnURL: str) -> dict:
