@@ -1,5 +1,5 @@
 from descope.auth import Auth
-from descope.common import EndpointsV1
+from descope.common import EndpointsV1, LoginOptions, validateRefreshTokenProvided
 from descope.exceptions import ERROR_TYPE_INVALID_ARGUMENT, AuthException
 
 
@@ -9,7 +9,13 @@ class SAML:
     def __init__(self, auth: Auth):
         self._auth = auth
 
-    def start(self, tenant: str, return_url: str = None) -> dict:
+    def start(
+        self,
+        tenant: str,
+        return_url: str = None,
+        loginOptions: LoginOptions = None,
+        refreshToken: str = None,
+    ) -> dict:
         """
         Docs
         """
@@ -23,9 +29,13 @@ class SAML:
                 400, ERROR_TYPE_INVALID_ARGUMENT, "Return url cannot be empty"
             )
 
+        validateRefreshTokenProvided(loginOptions, refreshToken)
+
         uri = EndpointsV1.authSAMLStart
         params = SAML._compose_start_params(tenant, return_url)
-        response = self._auth.do_get(uri, params)
+        response = self._auth.do_post(
+            uri, loginOptions.__dict__ if loginOptions else {}, params, refreshToken
+        )
 
         return response.json()
 
